//=======================================================================
//                        MicroBoard LX9 USBHOST
//
// File:
//    mig_wrapper.v
//
// Abstract:
//    Spartan6 mig wrapper
//
// Created:
//    16 June 2013
//
//======================================================================
//
// Copyright (c) 2013, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//  Revision History

module mig_wrapper #
(
   parameter C3_NUM_DQ_PINS          = 16,       
                                       // External memory data width
   parameter C3_MEM_ADDR_WIDTH       = 13,       
                                       // External memory address width
   parameter C3_MEM_BANKADDR_WIDTH   = 2        
                                       // External memory bank address width
)	
 (
  // LPDDR
  mcb3_dram_dq,
  mcb3_dram_a,
  mcb3_dram_ba,
  mcb3_dram_cke,
  mcb3_dram_ras_n,
  mcb3_dram_cas_n,
  mcb3_dram_we_n,
  mcb3_dram_dm,
  mcb3_dram_udqs,
  mcb3_rzq,
  mcb3_dram_udm,
  c3_sys_clk,
  c3_sys_rst_i,
  c3_calib_done,
  c3_clk0,
  c3_rst0,
  mcb3_dram_dqs,
  mcb3_dram_ck,
  mcb3_dram_ck_n,
  // cpu access
  clk_core,
  rst_x,
  // cpu access
  i_req_cpu,
  i_wr_cpu,
  i_adrs_cpu,
  o_ack_cpu,
  i_be_cpu,
  i_wd_cpu,
  o_rstr_cpu,
  o_rd_cpu,
  // dc access
  i_req_dc,
  i_adrs_dc,
  i_len_dc,
  o_ack_dc,
  o_rstr_dc,
  o_rd_dc,
  // debug
  c3_p0_rd_empty,
  c3_p0_cmd_full,
  c3_p0_wr_full
);

  inout  [C3_NUM_DQ_PINS-1:0] mcb3_dram_dq;
  output [C3_MEM_ADDR_WIDTH-1:0] mcb3_dram_a;
  output [C3_MEM_BANKADDR_WIDTH-1:0] mcb3_dram_ba;
  output mcb3_dram_cke;
  output mcb3_dram_ras_n;
  output mcb3_dram_cas_n;
  output mcb3_dram_we_n;
  output mcb3_dram_dm;
  inout  mcb3_dram_udqs;
  inout  mcb3_rzq;
  output mcb3_dram_udm;
  input  c3_sys_clk;
  input  c3_sys_rst_i;
  output c3_calib_done;
  output c3_clk0;
  output c3_rst0;
  inout  mcb3_dram_dqs;
  output mcb3_dram_ck;
  output mcb3_dram_ck_n;
  // system
  input clk_core;
  input rst_x;
  // cpu access
  input i_req_cpu;
  input i_wr_cpu;
  input [21:0] i_adrs_cpu;
  output o_ack_cpu;
  input [3:0] i_be_cpu;
  input [31:0] i_wd_cpu;
  output o_rstr_cpu;
  output [31:0] o_rd_cpu;
  // dc access
  input i_req_dc;
  input [21:0] i_adrs_dc;
  input [5:0] i_len_dc;
  output o_ack_dc;
  output o_rstr_dc;
  output [31:0] o_rd_dc;
  // debug
  output c3_p0_rd_empty;
  output c3_p0_cmd_full;
  output c3_p0_wr_full;

  wire w_c3_p0_rd_empty;
  wire w_c3_p0_cmd_full;
  wire w_c3_p0_wr_full;
  wire w_c3_p1_rd_empty;
  wire w_c3_p1_cmd_full;

  wire w_c3_p0_cmd_en;
  wire w_c3_p0_wr_en;
  wire w_wr_acceptable;
  assign c3_p0_rd_empty = w_c3_p0_rd_empty;
  assign c3_p0_cmd_full = w_c3_p0_cmd_full;
  assign c3_p0_wr_full = w_c3_p0_wr_full;

  assign w_c3_p0_cmd_en = (i_wr_cpu) ? i_req_cpu & w_wr_acceptable : i_req_cpu;
  assign w_c3_p0_wr_en = (i_wr_cpu) ? i_req_cpu & w_wr_acceptable : 1'b0;
  assign o_rstr_cpu = !w_c3_p0_rd_empty;
  assign w_wr_acceptable = !w_c3_p0_cmd_full & !w_c3_p0_wr_full;
  assign o_ack_cpu = (i_wr_cpu) ? w_wr_acceptable:
                                 !w_c3_p0_cmd_full;

  assign o_rstr_dc = !w_c3_p1_rd_empty;
  assign o_ack_dc = !w_c3_p1_cmd_full;

`ifdef BSIM
mig_39_2 #(.C3_SIMULATION("TRUE")) u_mig (
`else
mig_39_2 u_mig (
`endif
  .mcb3_dram_dq(mcb3_dram_dq),
  .mcb3_dram_a(mcb3_dram_a),
  .mcb3_dram_ba(mcb3_dram_ba),
  .mcb3_dram_cke(mcb3_dram_cke),
  .mcb3_dram_ras_n(mcb3_dram_ras_n),
  .mcb3_dram_cas_n(mcb3_dram_cas_n),
  .mcb3_dram_we_n(mcb3_dram_we_n),
  .mcb3_dram_dm(mcb3_dram_dm),
  .mcb3_dram_udqs(mcb3_dram_udqs),
  .mcb3_rzq(mcb3_rzq),
  .mcb3_dram_udm(mcb3_dram_udm),
  .c3_sys_clk(c3_sys_clk),
  .c3_sys_rst_i(c3_sys_rst_i),
  .c3_calib_done(c3_calib_done),
  .c3_clk0(c3_clk0),
  .c3_rst0(c3_rst0),
  .mcb3_dram_dqs(mcb3_dram_dqs),
  .mcb3_dram_ck(mcb3_dram_ck),
  .mcb3_dram_ck_n(mcb3_dram_ck_n),
  // cpu port
  .c3_p0_cmd_clk(clk_core),
  .c3_p0_cmd_en(w_c3_p0_cmd_en),
  .c3_p0_cmd_instr({2'b00,~i_wr_cpu}),
  .c3_p0_cmd_bl('d0),
  .c3_p0_cmd_byte_addr({i_adrs_cpu,2'b00}),
  .c3_p0_cmd_empty(),
  .c3_p0_cmd_full(w_c3_p0_cmd_full),
  .c3_p0_wr_clk(clk_core),
  .c3_p0_wr_en(w_c3_p0_wr_en),
  .c3_p0_wr_mask(~i_be_cpu),
  .c3_p0_wr_data(i_wd_cpu),
  .c3_p0_wr_full(w_c3_p0_wr_full),
  .c3_p0_wr_empty(),
  .c3_p0_wr_count(),
  .c3_p0_wr_underrun(),
  .c3_p0_wr_error(),
  .c3_p0_rd_clk(clk_core),
  .c3_p0_rd_en(!w_c3_p0_rd_empty),
  .c3_p0_rd_data(o_rd_cpu),
  .c3_p0_rd_full(),
  .c3_p0_rd_empty(w_c3_p0_rd_empty),
  .c3_p0_rd_count(),
  .c3_p0_rd_overflow(),
  .c3_p0_rd_error(),
  // dc port (read only)
  .c3_p1_cmd_clk(clk_core),
  .c3_p1_cmd_en(i_req_dc),
  .c3_p1_cmd_instr(3'b001),
  .c3_p1_cmd_bl(i_len_dc-1'b1),
  .c3_p1_cmd_byte_addr({i_adrs_dc,2'b00}),
  .c3_p1_cmd_empty(),
  .c3_p1_cmd_full(w_c3_p1_cmd_full),
  .c3_p1_wr_clk(clk_core),
  .c3_p1_wr_en(1'b0),
  .c3_p1_wr_mask(4'h0),
  .c3_p1_wr_data(32'h0),
  .c3_p1_wr_full(),
  .c3_p1_wr_empty(),
  .c3_p1_wr_count(),
  .c3_p1_wr_underrun(),
  .c3_p1_wr_error(),
  .c3_p1_rd_clk(clk_core),
  .c3_p1_rd_en(!w_c3_p1_rd_empty),
  .c3_p1_rd_data(o_rd_dc),
  .c3_p1_rd_full(),
  .c3_p1_rd_empty(w_c3_p1_rd_empty),
  .c3_p1_rd_count(),
  .c3_p1_rd_overflow(),
  .c3_p1_rd_error()
);

endmodule   

 
