#include <stdio.h>
#include <string.h>
#include "video.h"
//#include "util.h"
#include "usbh_env.h"



void buffer_clear_fb(unsigned int c,int bank) {
  int i;
  unsigned int a;

  if (bank==1) a = FB1_ADRS;
  else a = FB0_ADRS;
  for (i = a; i < a + 0x96000; i += 4 ) {
    (*(volatile unsigned int  *)i) = c;
  }
}

void put_pixel_fb(int x, int y, unsigned int c, int rev,int bank) {
  int p,a;

  p = y * 640 + x;  // linear pixel position
  if (bank==1) a = FB1_ADRS + p*2;
  else a = FB0_ADRS + p*2;  //
  (*(volatile unsigned short  *)a) = (rev == 1) ? ~c : c;
}


void draw_splite(int x, int y, int offset,unsigned short *sp,int rev) {
	  int i,j;
	  unsigned int wd;
	  int p,a;

	  for (i=0;i<16;i++) {
	    // V
	    for (j=0;j<16;j+=2) { // per 2 pixels
	  	  p = (y+i) * 640 + (x+j);  // linear pixel position

	      // H
	  	  if (fb_front==1) {
            a = FB1_ADRS + p*2;
	  	  }
	  	  else {
	  		a = FB0_ADRS + p*2;
	  	  }
	  	  a &= 0xfffffffc;
	  	  wd =   *(sp+offset+j+0+i*16*16) & 0xffff;
	  	  wd |= (*(sp+offset+j+1+i*16*16) & 0xffff) << 16;
	  	  (*(volatile unsigned int *)a) = (rev == 1) ? ~wd : wd;
	  	}
	  }
}

void draw_char(int x, int y, char c, unsigned short *sp,int rev) {
	 int offset=0;
	 // ascii -> offset conversion
	 if (c >= 'a' && c <= 'z') {
	   offset = ((c-'a'+65)/WIDTH)*WIDTH*XPS*YPS +
	             (((c-'a')%WIDTH) + 1)*XPS;
	 } else
	 if (c >= 'A' && c <= 'Z') {
	   offset = ((c-'A'+33)/WIDTH)*WIDTH*XPS*YPS +
	             (((c-'A')%WIDTH) + 1)*XPS;
	 } else
	 if (c >= ' ' && c <= '@') {
	   offset = ((c-' ')/WIDTH)*WIDTH*XPS*YPS +
	             ((c-' ')%WIDTH)*XPS;
	 }
	 draw_splite(x,y, offset,sp,rev);
}

void draw_string(int x, int y, char *s, int rev) {
  int i,len;
  len = strlen(s);
  for (i=0;i<len;i++) {
    draw_char(x+i*XPS, y, *(s+i), (unsigned short*)(FONT_BUFFER),rev);
  }
}


void video_init() {
  FB0_OFFSET = FB0_ADRS;
  FB1_OFFSET = FB1_ADRS;
  //xil_printf("fb0 %x\n",FB0_OFFSET);
  //xil_printf("fb1 %x\n",FB1_OFFSET);
  // Color mode 5:6:5
  COLOR_MODE = 0x00000000;
  fb_front = 0;
  FRONT_BUFFER = 0;
  VIDEO_INT_MASK = 2;
  // Video Start
  VIDEO_START = 0x00000001;
  // clear screen
  buffer_clear_fb(0x01e001e0,0);
  buffer_clear_fb(0x01e001e0,1);
  //buffer_clear_fb(0x001f001f,1);
  //
  gen_bitmap_data();
}

void gen_bitmap_data()  {
  int i,j,k;
  unsigned short *buf;
  int pbuf;
  char r,g,b;
  r = 0;
  g = 0x1f;
  b = 0;
  pbuf = 0;
  buf = (unsigned short*)FONT_BUFFER;
  for (i=0;i<16;i++) {
    for (j=0;j<16;j++) {
        if ((j%2)==0) g = 0x0f; else g = 0x7f;
    	for (k=0;k<16;k++) {
        // 565 conversion
        *(buf+pbuf) =  ((b & 0x1f)<<11) |((g & 0x3f)<<5)| (r & 0x1f);
        pbuf++;
      }
    }
  }
}

void update_usb_input() {

}

static void UlToStr(char *s, unsigned long bin, unsigned char n)
{
    s += n;
    *s = '\0';

    while (n--)
    {
        *--s = (((int)bin % 16)>9)?  ((int)bin % 16)+ 'A'-10 : ((int)bin % 16) + '0';
        bin /= 16;
    }
}

void video_update() {
  update_mouse_pointer();
  video_status = 0;
  while (!video_status);
  FRONT_BUFFER = fb_front;
  if (fb_front == 0) fb_front = 1;
  else fb_front = 0;
}
extern st_usbh_env usbh_env;
extern un_hid_mouse mouse_data;


void update_mouse_pointer() {
  int x,y,rev;
  static float px = 0.0;
  static float py = 0.0;
  // clear current
  //x = get_mouse_x(px);
  //y = get_mouse_y(py);
  //draw_char(x, y,' ', (unsigned short*)(FONT_BUFFER),0);
  buffer_clear_fb(0x01e001e0,fb_front);

  if ((char)mouse_data.BYTE.x < 0) px -= MOUSE_MOVE_DELTA;
  else if ((char)mouse_data.BYTE.x > 0) px += MOUSE_MOVE_DELTA;
  if ((char)mouse_data.BYTE.y < 0) py -= MOUSE_MOVE_DELTA;
  else if ((char)mouse_data.BYTE.y > 0) py += MOUSE_MOVE_DELTA;
  if (px < -1.0) px = -1.0;
  if (px >  1.0) px =  1.0;
  if (py < -1.0) py = -1.0;
  if (py >  1.0) py =  1.0;

  if ((char)mouse_data.BYTE.b != 0) {
    xil_printf("mouse button pushed\n");
	rev = 1;
  }
  else rev = 0;
  x = get_mouse_x(px);
  y = get_mouse_y(py);
  draw_char(x, y,'!', (unsigned short*)(FONT_BUFFER),rev);

  mouse_data.BYTE.x = 0;
  mouse_data.BYTE.y = 0;
}

int get_mouse_x(float px) {
  return (int)(((px+1.0)/2.0)*(float)MOUSE_SCREEN_WIDTH) + MOUSE_SCREEN_OFFSET_X;
}
int get_mouse_y(float py){
  return (int)(((py+1.0)/2.0)*(float)MOUSE_SCREEN_HEIGHT) + MOUSE_SCREEN_OFFSET_Y;
}
