#ifndef __VIDEO__
#define __VIDEO__

#define FB0_ADRS     0xe0000000
#define FB1_ADRS     0xe0100000
#define FONT_BUFFER  0xe0200000
#define KEY_BUFFER   0xe0300000

#define VIDEO_START (*(volatile unsigned int  *)0xd0000000)
#define FB0_OFFSET  (*(volatile unsigned int  *)0xd0000004)
#define FB1_OFFSET  (*(volatile unsigned int  *)0xd0000008)
#define COLOR_MODE  (*(volatile unsigned int  *)0xd000000c)
#define VIDEO_STATUS      (*(volatile unsigned int  *)0xd0000024)
#define VIDEO_INT_MASK (*(volatile unsigned int  *)0xd0000028)
#define FRONT_BUFFER (*(volatile unsigned int  *)0xd000002c)

// font size
#define XPS 16
#define YPS 16
#define WIDTH 16
#define HEIGHT 16
// mouse screen
#define MOUSE_MOVE_DELTA 0.075
#define MOUSE_SCREEN_OFFSET_X 0
#define MOUSE_SCREEN_OFFSET_Y 0
#define MOUSE_SCREEN_WIDTH (640 - XPS)
#define MOUSE_SCREEN_HEIGHT (480-YPS)
typedef union {
  unsigned char BYTE[14];
  struct {
    unsigned short bfType;
    unsigned short bfSize[2];
    unsigned short bfReserved1;
    unsigned short bfReserved2;
    unsigned short bfOffBits[2];
  } KIND;
} un_bitmap_file_header;

typedef union {
  unsigned char BYTE[12];
  struct {
    unsigned short bcSize[2];
    short bcWidth;
    short bcHeight;
    unsigned short bcPlanes;
    unsigned short bcBitCount;
  } KIND;
} un_bitmap_core_header;

typedef union {
  unsigned char BYTE[40];
  struct {
    unsigned long  biSize;
    long           biWidth;
    long           biHeight;
    unsigned short biPlanes;
    unsigned short biBitCount;
    unsigned long  biCompression;
    unsigned long  biSizeImage;
    long           biXPixPerMeter;
    long           biYPixPerMeter;
    unsigned long  biClrUsed;
    unsigned long  biClrImporant;
  } KIND;
} un_bitmap_info_header;

typedef struct {
  unsigned char PACKED[8];
  int index_x;
  int index_y;
  int index_button;
  struct {
    unsigned char x;             // left: 0, center:0x7f. right: 0xff
    unsigned char y;             // up: 0, center:0x7f. down: 0xff
    union {
      unsigned char BYTE;
      struct {
        unsigned char button0 : 1;
        unsigned char button1 : 1;
        unsigned char button2 : 1;
        unsigned char button3 : 1;
        unsigned char button4 : 1;
        unsigned char button5 : 1;
        unsigned char button6 : 1;
        unsigned char button7 : 1;
      } BIT;
    } b0;
    union {
      unsigned char BYTE;
      struct {
        unsigned char button0 : 1;
        unsigned char button1 : 1;
        unsigned char         : 6;
      } BIT;
    } b1;
  } BYTE;
} st_hid_gamepad;

typedef union {
  unsigned char PACKED[3];
  struct {
	  /*
    union {
      unsigned char BYTE;
      struct {
        unsigned char button1 : 1;
        unsigned char button2 : 1;
        unsigned char button3 : 1;
        unsigned char         : 5;
      } BIT;
    } b;
    */
    unsigned char b;
	unsigned char x;             // left: 0, center:0x7f. right: 0xff
    unsigned char y;             // up: 0, center:0x7f. down: 0xff
  } BYTE;
} un_hid_mouse;
typedef union {
  unsigned char PACKED[8];
  struct {
    unsigned char modifier;
    unsigned char dummy;
    unsigned char keycode[6];
  } BYTE;
} un_hid_keyboard;

void buffer_clear_fb(unsigned int c, int bank);
void put_pixel_fb(int x, int y, unsigned int c,int rev,int bank);
void draw_pic(int x, int y, int xs, int ys, unsigned short *sp);
void draw_splite(int x, int y, int offset,unsigned short *sp,int rev);
void draw_char(int x, int y, char c, unsigned short *sp,int rev);

void draw_string(int x, int y,char *s,int rev);
void gen_bitmap_data();
void video_init();
void video_update();
void update_usb_tree();
void update_usb_input();
void update_mouse_pointer();
int get_mouse_x(float px);
int get_mouse_y(float py);
void draw_dir();

extern volatile int fb_front;
extern volatile int video_status;

#endif

