/*=======================================================================
// USB Host Library
//
// File:
//   video.h
//
// Title:
//   USB Host sample application video function body
//
// Created:
//     29 June 2013
//
//======================================================================*/
//
// Copyright (c) 2013, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//  Revision History

#include <stdio.h>
#include <string.h>
#include "video.h"
#include "usbh_env.h"
#include "pff.h"

extern FATFS fatfs;

static void UlToStr(char *s, unsigned long bin, unsigned char n)
{
    s += n;
    *s = '\0';

    while (n--)
    {
        *--s = (((int)bin % 16)>9)?  ((int)bin % 16)+ 'A'-10 : ((int)bin % 16) + '0';
        bin /= 16;
    }
}

static void UlToStr10(char *s, unsigned long bin, unsigned char n)
{
    s += n;
    *s = '\0';

    while (n--)
    {
        *--s = ((int)bin % 10) + '0';
        bin /= 10;
    }
}

static void die (       /* Stop with dying message */
        FRESULT rc      /* FatFs return value */
)
{
  switch(rc) {
  case FR_DISK_ERR:     /* 1 */
          xil_printf("\nFailed with FR_DISK_ERR.\n");
          break;
  case FR_NOT_READY:    /* 2 */
          xil_printf("\nFailed with FR_NOT_READY.\n");
          break;
  case FR_NO_FILE:      /* 3 */
          xil_printf("\nFailed with FR_NO_FILE.\n");
          break;
  case FR_NO_PATH:      /* 4 */
          xil_printf("\nFailed with FR_NO_PATH.\n");
          break;
  case FR_NOT_OPENED:   /* 5 */
          xil_printf("\nFailed with FR_NOT_OPENED.\n");
          break;
  case FR_NOT_ENABLED:  /* 6 */
          xil_printf("\nFailed with FR_NOT_ENABLED.\n");
          break;
  case FR_NO_FILESYSTEM:/* 7 */
          xil_printf("\nFailed with FR_NO_FILESYSTEM.\n");
          break;

  }

  for (;;) ;
}
void get_bitmap_data(char *file_name)  {
  unsigned short *buf;
  FRESULT rc;
  DIR dir;
  FILINFO fno;
  WORD bw, br;
  int i,j,pbuf;
  unsigned char r,g,b,a;
  un_bitmap_file_header bfh;
  un_bitmap_info_header bih;
  buf = (unsigned short*)FONT_BUFFER;
  pbuf = 0;

  pf_mount(&fatfs);
  rc = pf_open(file_name);

  if (rc) die(rc);
  rc = pf_read(bfh.BYTE, sizeof(bfh), &br);
  if (rc) die(rc);
  rc = pf_read(bih.BYTE, sizeof(bih), &br);
  if (rc) die(rc);
  xil_printf("loading bitmap size w,h = %d %d\n",bih.KIND.biWidth,bih.KIND.biHeight);
  for (j=0;j<bih.KIND.biHeight;j++) {
    for (i=0;i<bih.KIND.biWidth;i++) {
      // 565 conversion
      rc = pf_read(&b, 1, &br);
      rc = pf_read(&g, 1, &br);
      rc = pf_read(&r, 1, &br);
      if (bih.KIND.biBitCount==32) rc = pf_read(&a, 1, &br);
      *(volatile unsigned short *)(FONT_BUFFER+pbuf) =  (((r>>3) & 0x1f)<<11) |(((g>>2) & 0x3f)<<5)| ((b>>3) & 0x1f);
      pbuf +=2;
    }
  }

  xil_printf("done\n");
  if (rc) die(rc);

}

void draw_dir() {
  FRESULT rc;    /* Result code */
  DIR dir;       /* Directory object */
  FILINFO fno;   /* File information object */
  WORD bw, br;
  char s[256];
  int i,j;
  int offset = 288;

  if (!uh_msd_available(0)) return;
  xil_printf("\nOpen root directory.\n");
  rc = pf_opendir(&dir, "");
  if (rc) die(rc);
  xil_printf("\nDirectory listing...\n");
  for (j=0;j<12;j++) {
      for (i=0;i<256;i++) s[i] = 0;       /* width limit */
      rc = pf_readdir(&dir, &fno);        /* Read a directory item */
      if (rc || !fno.fname[0]) break;     /* Error or end of dir */
      if (fno.fattrib & AM_DIR) {
      xil_printf("  <dir>  %s\n", fno.fname);
      draw_string(XPS,offset,"<dir>  ",0);
      for (i=0;i<10;i++) s[i] = fno.fname[i];
      draw_string(8*XPS,offset,s,0);
  } else {
      xil_printf("%8d  %s\n", fno.fsize, fno.fname);
      UlToStr10(s,fno.fsize, 4);
      draw_string(XPS,offset,s,0);
          for (i=0;i<10;i++) s[i] = fno.fname[i];
      draw_string(8*XPS,offset,s,0);
    }
    offset += YPS;
  }
  if (rc) die(rc);

}

void buffer_clear_fb(unsigned int c,int bank) {
  int i;
  unsigned int a;

  if (bank==1) a = FB1_ADRS;
  else a = FB0_ADRS;
  for (i = a; i < a + SCREEN_W*SCREEN_H*2/* 640*480*2 */; i += 4 ) {
    (*(volatile unsigned int  *)i) = c;
  }
}

void buffer_clear_fb_bk(int sy, int es,int xoffset,unsigned int c,int bank) {
  int i;
  unsigned int sa,ea;
  int xo;
  //sy : start y position
  //es : fill y size
  //xoffset : x pixel position offset, should be 32bit aligned
  if (bank==1) sa = FB1_ADRS;
  else sa = FB0_ADRS;
  sa = sa + sy * SCREEN_W * 2;
  ea = sa + es * SCREEN_W * 2;
  xo=0;
  for (i = sa; i < ea; i += 4 ) {
    if (xo> xoffset)
          (*(volatile unsigned int  *)i) = c;  // 2 pixel colors
    xo += 2;
    if (xo >= SCREEN_W) {
      xo = 0;
    }
  }
}


void draw_splite(int x, int y, int offset,unsigned short *sp,int rev) {
    int i,j;
    unsigned int wd;
    int p,a;

    for (i=0;i<16;i++) {
        // V
        for (j=0;j<16;j+=2) { // per 2 pixels
            p = (y+i) * 640 + (x+j);  // linear pixel position
            // H
            if (fb_front==1) {
                a = FB1_ADRS + p*2;
            }
            else {
                a = FB0_ADRS + p*2;
            }
                a &= 0xfffffffc;
                wd =   *(sp+offset+j+0+i*16*16) & 0xffff;
                wd |= (*(sp+offset+j+1+i*16*16) & 0xffff) << 16;
                (*(volatile unsigned int *)a) = (rev == 1) ? ~wd : wd;
            }
      }
}

void show_mem(int x, int y,int adr) {
    int i,j;
    unsigned int wd;
    int p,a;
    xil_printf("adr = %08x\n",adr);
    for (i=0;i<16;i++) {
        // V
        for (j=0;j<16;j+=2) { // per 2 pixels
            p = (y+i) * 640 + (x+j);  // linear pixel position
            // H
            a = adr + p*2;
            a &= 0xfffffffc;
            xil_printf("y,x = %x %x a = %08x mem = %08x\n",i,j,a,(*(volatile unsigned int *)a));
        }
    }
}
void show_f(int x, int y,int adr) {
    int i,j;
    unsigned int wd;
    int p,a;
    xil_printf("adr = %08x\n",adr);
    for (i=0;i<16;i++) {
        // V
        for (j=0;j<16;j+=2) { // per 2 pixels
            p = (y+i) * 256 + (x+j);  // linear pixel position
            // H
            a = adr + p*2;
            a &= 0xfffffffc;
            xil_printf("y,x = %x %x a = %08x mem = %08x\n",i,j,a,(*(volatile unsigned int *)a));
        }
    }
}

void show_buffer(int bank) {
    int i,j=0;
    unsigned int a;

    if (bank==1) a = FB1_ADRS;
    else a = FB0_ADRS;
    for (i = a; i < a + SCREEN_W*SCREEN_H*2/* 640*480*2 */; i += 4 ) {
        xil_printf("0x%08x,",(*(volatile unsigned int  *)i));
        if ((j % 8)==0)xil_printf("\n");
        j++;
    }
}

void draw_char(int x, int y, char c, unsigned short *sp,int rev) {
    int offset;
    // ascii -> offset conversion
    if (c >= 'a' && c <= 'z') {
        offset = ((c-'a'+65)/WIDTH)*WIDTH*XPS*YPS +
                 (((c-'a')%WIDTH) + 1)*XPS;
    } else
    if (c >= 'A' && c <= 'Z') {
        offset = ((c-'A'+33)/WIDTH)*WIDTH*XPS*YPS +
                 (((c-'A')%WIDTH) + 1)*XPS;
    } else
    if (c >= ' ' && c <= '@') {
        offset = ((c-' ')/WIDTH)*WIDTH*XPS*YPS +
                 ((c-' ')%WIDTH)*XPS;
    }
    draw_splite(x,y, offset,sp,rev);
}

void draw_string(int x, int y, char *s, int rev) {
    int i,len;
    len = strlen(s);
    for (i=0;i<len;i++) {
        draw_char(x+i*XPS, y, *(s+i), (unsigned short*)(FONT_BUFFER),rev);
    }
}


void video_init() {
    FB0_OFFSET = FB0_ADRS;
    FB1_OFFSET = FB1_ADRS;
    // Color mode 5:6:5
    COLOR_MODE = 0x00000000;
    FRONT_BUFFER = 0;
    VIDEO_INT_MASK = 2;
    // Video Start
    VIDEO_START = 0x00000001;
    // clear screen
    buffer_clear_fb(0x00000000,0);
    buffer_clear_fb(0x00000000,1);
    // 
}


#if 0
void video_update() {
    static int iflag = 0;

    if (iflag < 2) {
        buffer_clear_fb(0x00000000,fb_front);
        draw_string(XPS,YPS,"**** MICROBOARD USB host demo ******",0);
        draw_dir();  // draw both screen
    } else {
        // clear upper half area
        buffer_clear_fb_bk(32,MOUSE_SCREEN_OFFSET_Y,0,
                               0x00000000,fb_front);
        // clear bottom right area
        buffer_clear_fb_bk(MOUSE_SCREEN_OFFSET_Y,SCREEN_H-MOUSE_SCREEN_OFFSET_Y,MOUSE_SCREEN_OFFSET_X-8,
                               0x00000000,fb_front);
    }
    uh_update();

    update_usb_tree();
    update_usb_input();
    update_mouse_pointer();

    uh_update();
    video_status = 0;
    while (!video_status);
    FRONT_BUFFER = fb_front;
    if (fb_front == 0) fb_front = 1;
    else fb_front = 0;
    if (iflag < 3) iflag++;
}
#else
void video_update() {
    static int iflag = 0;
    static int func_num = 0;

    switch (func_num) {
    case 0: // clear buffer
        if (iflag < 2) {
            buffer_clear_fb(0x00000000,fb_front);
            draw_string(XPS,YPS,"**** MICROBOARD USB host demo ******",0);
            draw_dir();  // draw both screen
        } else {
            // clear upper half area
            buffer_clear_fb_bk(32,MOUSE_SCREEN_OFFSET_Y,0,
                                   0x00000000,fb_front);
            // clear bottom right area
            buffer_clear_fb_bk(MOUSE_SCREEN_OFFSET_Y,SCREEN_H-MOUSE_SCREEN_OFFSET_Y,MOUSE_SCREEN_OFFSET_X-8,
                                   0x00000000,fb_front);
        }
        func_num = 1;
        break;
    case 1:
        update_usb_tree();
        func_num = 2;
        break;
    case 2:
        update_mouse_pointer();
        func_num = 3;
        break;
    case 3:  // frame buffer swap
    	video_status = 0;
    	while (!video_status);
    	FRONT_BUFFER = fb_front;
    	if (fb_front == 0) fb_front = 1;
    	else fb_front = 0;
    	if (iflag < 3) iflag++;
    	func_num = 0;
    	break;
    }
}

#endif
extern st_usbh_env usbh_env;
extern st_hid_gamepad gamepad_data;
extern un_hid_mouse mouse_data;
extern un_hid_keyboard keyboard_data;

void update_usb_tree() {

    int offset,len,i;
    char c,s[8];
    volatile char *kb;
    kb = (volatile char *)(KEY_BUFFER);

    offset = 3*YPS+0*7*YPS;
    if (um_is_hub(usbh_env.conn[0].p_root)) {
        draw_string(0,offset+YPS,"  HUB Status Change Bitmap: ",0);
        UlToStr(s,usbh_env.hub_status , 2);
        draw_string(30*XPS,offset+YPS,s,0);
        for (i=1;i<=usbh_env.conn[0].p_root->descs.hub.BYTE.bNbrPorts;i++) {
            draw_string(0,offset+(i+1)*YPS,"  Port ",0);
            UlToStr(s,i, 2);
            draw_string(8*XPS,offset+(i+1)*YPS,s,0);
            if ((usbh_env.conn[0].p_root->p_devices[i] != NULL)&&
                 usbh_env.conn[0].p_root->p_devices[i]->info.available) {
            	if (um_is_hub(usbh_env.conn[0].p_root->p_devices[i])) {
             	  draw_string(10*XPS,offset+(i+1)*YPS,": HUB",0);
            	} else if (usbh_env.conn[0].p_root->p_devices[i]->descs.interface[0].BYTE.bInterfaceClass == UCC_MASS_STRAGE) {
                     draw_string(10*XPS,offset+(i+1)*YPS,": Mass Storage Device",0);
                } else {
                    switch(usbh_env.conn[0].p_root->p_devices[i]->descs.interface[0].BYTE.bInterfaceProtocol) {
                    default:
                        draw_string(10*XPS,offset+(i+1)*YPS,": not available    ",0);
                        break;
                    case 0:
                        //sprintf(s,"  Port %d: Game Pad X=%02x Y=%02x B0=%02x B1=%02x",i,
                        //gamepad_data.BYTE.x,
                        //gamepad_data.BYTE.y,
                        //gamepad_data.BYTE.b0.BYTE,
                        //gamepad_data.BYTE.b1.BYTE
                        //);
                        draw_string(10*XPS,offset+(i+1)*YPS,": Game Pad X,Y,B=",0);
                        UlToStr(s,gamepad_data.BYTE.x, 2);
                        draw_string(27*XPS,offset+(i+1)*YPS,s,0);
                        UlToStr(s,gamepad_data.BYTE.y, 2);
                        draw_string(30*XPS,offset+(i+1)*YPS,s,0);
                        UlToStr(s,gamepad_data.BYTE.b0.BYTE, 2);
                        draw_string(33*XPS,offset+(i+1)*YPS,s,0);
                        UlToStr(s,gamepad_data.BYTE.b1.BYTE, 2);
                        draw_string(36*XPS,offset+(i+1)*YPS,s,0);
                        break;
                    case 1:
                        len = strlen(kb);
                        if (keyboard_data.BYTE.keycode[0] == 0x7f) {
                            draw_string((21+len)*XPS,offset+(i+1)*YPS," ",0);
                        }
                        draw_string(10*XPS,offset+(i+1)*YPS,": Keyboard",0);
                        draw_string(21*XPS,offset+(i+1)*YPS,kb,0);
                        break;
                    case 2:
                        //sprintf(s,"  Port %d: Mouse X=%02x Y=%02x B=%02x",i,
                        //  mouse_data.BYTE.x,
                        //  mouse_data.BYTE.y,
                        //  mouse_data.BYTE.b.BYTE
                        //  );
                        draw_string(10*XPS,offset+(i+1)*YPS,": Mouse X, Y, B =",0);
                        UlToStr(s,mouse_data.BYTE.x, 2);
                        draw_string(30*XPS,offset+(i+1)*YPS,s,0);
                        UlToStr(s,mouse_data.BYTE.y, 2);
                        draw_string(33*XPS,offset+(i+1)*YPS,s,0);
                        UlToStr(s,mouse_data.BYTE.b, 2);
                        draw_string(36*XPS,offset+(i+1)*YPS,s,0);
                        break;
                    }
                }
            } else {
                //sprintf(s,"  Port %d: not available                 ",i);
                draw_string(10*XPS,offset+(i+1)*YPS,": not available",0);
            }
        }
    }
}

void update_mouse_pointer() {
    int x,y,rev;
    static float px = 0.0;
    static float py = 0.0;
    // clear current
    x = get_mouse_x(px);
    y = get_mouse_y(py);
    //draw_char(x, y,' ', (unsigned short*)(FONT_BUFFER),0);

    if ((char)mouse_data.BYTE.x < 0) px -= MOUSE_MOVE_DELTA;
    else if ((char)mouse_data.BYTE.x > 0) px += MOUSE_MOVE_DELTA;
    if ((char)mouse_data.BYTE.y < 0) py -= MOUSE_MOVE_DELTA;
    else if ((char)mouse_data.BYTE.y > 0) py += MOUSE_MOVE_DELTA;
    if (px < -1.0) px = -1.0;
    if (px >  1.0) px =  1.0;
    if (py < -1.0) py = -1.0;
    if (py >  1.0) py =  1.0;

   if (mouse_data.BYTE.b != 0) {
	  xil_printf("mouse button pushed\n");
      rev = 1;
   }
   else rev = 0;
   // clear current
   x = get_mouse_x(px);
   y = get_mouse_y(py);
   draw_char(x, y,'o', (unsigned short*)(FONT_BUFFER),rev);

   mouse_data.BYTE.x = 0;
   mouse_data.BYTE.y = 0;
}

int get_mouse_x(float px) {
    return (int)(((px+1.0)/2.0)*(float)MOUSE_SCREEN_WIDTH) + MOUSE_SCREEN_OFFSET_X;
}
int get_mouse_y(float py){
    return (int)(((py+1.0)/2.0)*(float)MOUSE_SCREEN_HEIGHT) + MOUSE_SCREEN_OFFSET_Y;
}
